==> spirit_hub-0.0.1.8.tar.gz: spirit_hub-0.0.1.8/spec/services/spirit_hub/tagset_send_to_users_spec.rb <==
# frozen_string_literal: true

require 'rails_helper'

RSpec.describe SpiritHub::TagsetSendToUsers, type: :service do
  before do
    SpiritHub::HubdocCycle.import_cycle_from_json
  end

  describe 'ArgumentError' do
    it 'raises an error if state_key is missing' do
      expect do
        described_class.call({})
      end.to raise_error(ArgumentError, 'State key is required')
    end

    it 'raises an error if state_key is invalid' do
      expect do
        described_class.call(state_key: :other_state)
      end.to raise_error(ArgumentError) do |e| # rubocop:disable Style/MultilineBlockChain
        expect(e.message).to include('Invalid state_key: other_state.')
      end
    end
  end

  describe 'validate/refuse' do
    let!(:user1) { create(:user) }
    let!(:user2) { create(:user) }
    let!(:tagset) { create(:tagset_doc) }

    before do
      SpiritHub::HubdocCycle.change_state(tagset, :to_validate, user1)
      tagset.save
    end

    subject { described_class.call(state_key:, tagset:, current_user: user1, recipients: [user2.id], message: 'lol') }

    context 'when the state is validate' do
      let!(:state_key) { :validate }

      it 'changes the state to validate and give it to other user' do
        subject
        LogJob.new.perform

        expect(tagset.reload.state.key).to eq('validate')
        expect(tagset.reload.current_state).to eq('Validé')
        expect(tagset.reload.accounted).to eq([user2.id.to_s])
        expect(tagset.logs.first.code).to eq('DOC_VALIDATED')
      end
    end

    context 'when the state is refuse' do
      let!(:state_key) { :refuse }

      it 'changes the state to refuse and give it to other user' do
        subject
        LogJob.new.perform

        expect(tagset.reload.state.key).to eq('refuse')
        expect(tagset.reload.current_state).to eq('Refusé')
        expect(tagset.reload.accounted).to eq([user2.id.to_s])
        expect(tagset.logs.first.code).to eq('DOC_REFUSED')
      end
    end
  end

  RSpec.shared_examples 'send_to_users' do |state_key, current_state|
    describe "send to users #{state_key}" do
      let!(:user1) { create(:user) }
      let!(:user2) { create(:user) }
      let!(:group) { create(:group, name: 'Groupe 1') }
      let!(:tagset) { create(:tagset_doc) }

      it 'works' do
        described_class.call(state_key:, tagset:, current_user: user1, recipients: [user2.id.to_s, "g_#{group.id}"], message: 'lol')
        LogJob.new.perform

        expect(tagset.reload.current_state).to eq(current_state)
        expect(tagset.reload.accounted).to eq([user2.id.to_s, "g_#{group.id}"])
        expect(tagset.logs.first.code).to eq("DOC_SENT_#{state_key.to_s.upcase}")
      end
    end
  end

  include_examples 'send_to_users', :to_share, 'À partager'
  include_examples 'send_to_users', :to_validate, 'À valider'
  include_examples 'send_to_users', :to_complete, 'À compléter'
  include_examples 'send_to_users', :to_answer, 'À répondre'
end

==> spirit_hub-0.0.1.11.tar.gz: spirit_hub-0.0.1.11/spec/services/spirit_hub/tagset_send_to_users_spec.rb <==
# frozen_string_literal: true

require 'rails_helper'

RSpec.describe SpiritHub::TagsetSendToUsers, type: :service do
  before do
    SpiritHub::HubdocCycle.import_cycle_from_json
  end

  describe 'ArgumentError' do
    it 'raises an error if state_key is missing' do
      expect do
        described_class.call({})
      end.to raise_error(ArgumentError, 'State key is required')
    end

    it 'raises an error if state_key is invalid' do
      expect do
        described_class.call(state_key: :other_state)
      end.to raise_error(ArgumentError) do |e| # rubocop:disable Style/MultilineBlockChain
        expect(e.message).to include('Invalid state_key: other_state.')
      end
    end
  end

  describe 'validate/refuse' do
    let!(:user1) { create(:user) }
    let!(:user2) { create(:user) }
    let!(:tagset) { create(:tagset_doc) }

    before do
      SpiritHub::HubdocCycle.change_state(tagset, :to_validate, user1)
      tagset.save
    end

    subject { described_class.call(state_key:, tagset:, current_user: user1, recipients: [user2.id], message: 'lol') }

    context 'when the state is validate' do
      let!(:state_key) { :validate }

      it 'changes the state to validate and give it to other user' do
        subject

        expect(tagset.reload.state.key).to eq('validate')
        expect(tagset.reload.current_state).to eq('Validé')
        expect(tagset.reload.accounted).to eq([user2.id.to_s])
        expect(SpiritHub::JournalEntry.last.code).to eq('DOC_VALIDATED')
      end
    end

    context 'when the state is refuse' do
      let!(:state_key) { :refuse }

      it 'changes the state to refuse and give it to other user' do
        subject
        expect(tagset.reload.state.key).to eq('refuse')
        expect(tagset.reload.current_state).to eq('Refusé')
        expect(tagset.reload.accounted).to eq([user2.id.to_s])
        expect(SpiritHub::JournalEntry.last.code).to eq('DOC_REFUSED')
      end
    end
  end

  RSpec.shared_examples 'send_to_users' do |state_key, current_state|
    describe "send to users #{state_key}" do
      let!(:user1) { create(:user) }
      let!(:user2) { create(:user) }
      let!(:group) { create(:group, name: 'Groupe 1') }
      let!(:tagset) { create(:tagset_doc) }

      it 'works' do
        described_class.call(state_key:, tagset:, current_user: user1, recipients: [user2.id.to_s, "g_#{group.id}"], message: 'lol')
        expect(tagset.reload.current_state).to eq(current_state)
        expect(tagset.reload.accounted).to eq([user2.id.to_s, "g_#{group.id}"])
        expect(SpiritHub::JournalEntry.last.code).to eq("DOC_SENT_#{state_key.to_s.upcase}")
      end
    end
  end

  include_examples 'send_to_users', :to_share, 'À partager'
  include_examples 'send_to_users', :to_validate, 'À valider'
  include_examples 'send_to_users', :to_complete, 'À compléter'
  include_examples 'send_to_users', :to_answer, 'À répondre'
end
